@@ -76,8 +76,11 @@ def session_query(session, model):
     """
     if hasattr(model, 'query'):
         if callable(model.query):
-            return model.query()
-        return model.query
+            query = model.query()
+        else:
+            query = model.query
+        if hasattr(query, 'filter'):
+            return query
     return session.query(model)
 
 
@@ -118,10 +121,10 @@ def get_related_model(model, relationname):
     whose name is `relationname`.
 
     """
-    cols = model._sa_class_manager
-    attr = getattr(model, relationname)
-    if relationname in cols and isinstance(attr.property, RelProperty):
-        return cols[relationname].property.mapper.class_
+    if hasattr(model, relationname):
+        attr = getattr(model, relationname)
+        if hasattr(attr, 'property') and isinstance(attr.property, RelProperty):
+            return attr.property.mapper.class_
     if isinstance(attr, AssociationProxy):
         return get_related_association_proxy_model(attr)
     return None
@@ -234,6 +237,10 @@ def is_like_list(instance, relation):
     """
     if relation in instance._sa_class_manager:
         return instance._sa_class_manager[relation].property.uselist
+    elif hasattr(instance, relation):
+        attr = getattr(instance._sa_instance_state.class_, relation)
+        if hasattr(attr, 'property'):
+            return attr.property.uselist
     related_value = getattr(type(instance), relation, None)
     if isinstance(related_value, AssociationProxy):
         local_prop = related_value.local_attr.prop
@@ -305,7 +312,8 @@ def to_dict(instance, deep=None, exclude=None, include=None,
         column_attrs = inspected_instance.column_attrs.keys()
         descriptors = inspected_instance.all_orm_descriptors.items()
         hybrid_columns = [k for k, d in descriptors
-                          if d.extension_type == hybrid.HYBRID_PROPERTY]
+                          if d.extension_type == hybrid.HYBRID_PROPERTY
+                          and not (deep and k in deep)]
         columns = column_attrs + hybrid_columns
     except NoInspectionAvailable:
         return instance
@@ -561,6 +569,7 @@ def count(session, query):
     queries.
 
     """
-    counts = query.statement.with_only_columns([func.count()])
+    num_results = None
+    counts = query.selectable.with_only_columns([func.count()])
     num_results = session.execute(counts.order_by(None)).scalar()
-    return query.count() if num_results is None else num_results
+    return query.count() if num_results is None or query._limit else num_results
