@@ -41,7 +41,9 @@ from sqlalchemy.exc import ProgrammingError
 from sqlalchemy.orm.exc import MultipleResultsFound
 from sqlalchemy.orm.exc import NoResultFound
 from sqlalchemy.orm.query import Query
+from werkzeug.exceptions import BadRequest
 from werkzeug.exceptions import HTTPException
+from werkzeug.urls import url_quote_plus
 
 from .helpers import count
 from .helpers import evaluate_functions
@@ -59,8 +61,6 @@ from .helpers import session_query
 from .helpers import strings_to_dates
 from .helpers import to_dict
 from .helpers import upper_keys
-from werkzeug.exceptions import BadRequest
-
 from .search import create_query
 from .search import search
 
@@ -139,7 +139,7 @@ def catch_processing_exceptions(func):
             return func(*args, **kw)
         except ProcessingException as exception:
             current_app.logger.exception(str(exception))
-            status, message = exception.code, str(exception)
+            status, message = exception.code, exception.description or str(exception)
             return jsonify(message=message), status
     return decorator
 
@@ -1017,7 +1017,7 @@ class API(ModelView):
                                             self._compute_results_per_page())
             headers = dict(Link=linkstring)
         else:
-            primary_key = primary_key_name(result)
+            primary_key = self.primary_key or primary_key_name(result)
             result = to_dict(result, deep, exclude=self.exclude_columns,
                              exclude_relations=self.exclude_relations,
                              include=self.include_columns,
@@ -1091,9 +1091,11 @@ class API(ModelView):
         """Removes the specified instance of the model with the specified name
         from the database.
 
-        Since :http:method:`delete` is an idempotent method according to the
-        :rfc:`2616`, this method responds with :http:status:`204` regardless of
-        whether an object was deleted.
+        Although :http:method:`delete` is an idempotent method according to
+        :rfc:`2616`, idempotency only means that subsequent identical requests
+        cannot have additional side-effects. Since the response code is not a
+        side effect, this method responds with :http:status:`204` only if an
+        object is deleted, and with :http:status:`404` when nothing is deleted.
 
         If `relationname
 
@@ -1122,13 +1124,14 @@ class API(ModelView):
                                        relationinstid)
             # Removes an object from the relation list.
             relation.remove(relation_instance)
+            is_deleted = len(self.session.dirty) > 0
         elif inst is not None:
             self.session.delete(inst)
-            self.session.commit()
-            is_deleted = True
+            is_deleted = len(self.session.deleted) > 0
+        self.session.commit()
         for postprocessor in self.postprocessors['DELETE']:
             postprocessor(is_deleted=is_deleted)
-        return {}, 204
+        return {}, 204 if is_deleted else 404
 
     def post(self):
         """Creates a new instance of a given model based on request data.
@@ -1166,20 +1169,20 @@ class API(ModelView):
             # HACK Requests made from Internet Explorer 8 or 9 don't have the
             # correct content type, so request.get_json() doesn't work.
             if is_msie:
-                params = json.loads(request.get_data()) or {}
+                data = json.loads(request.get_data()) or {}
             else:
-                params = request.get_json() or {}
+                data = request.get_json() or {}
         except (BadRequest, TypeError, ValueError, OverflowError) as exception:
             current_app.logger.exception(str(exception))
             return dict(message='Unable to decode data'), 400
 
         # apply any preprocessors to the POST arguments
         for preprocessor in self.preprocessors['POST']:
-            preprocessor(data=params)
+            preprocessor(data=data)
 
         # Check for any request parameter naming a column which does not exist
         # on the current model.
-        for field in params:
+        for field in data:
             if not has_field(self.model, field):
                 msg = "Model does not have field '{0}'".format(field)
                 return dict(message=msg), 400
@@ -1190,25 +1193,25 @@ class API(ModelView):
 
         # Looking for what we're going to set on the model right now
         colkeys = cols.keys()
-        paramkeys = params.keys()
+        paramkeys = data.keys()
         props = set(colkeys).intersection(paramkeys).difference(relations)
 
         # Special case: if there are any dates, convert the string form of the
         # date into an instance of the Python ``datetime`` object.
-        params = strings_to_dates(self.model, params)
+        data = strings_to_dates(self.model, data)
 
         try:
             # Instantiate the model with the parameters.
-            modelargs = dict([(i, params[i]) for i in props])
+            modelargs = dict([(i, data[i]) for i in props])
             instance = self.model(**modelargs)
 
             # Handling relations, a single level is allowed
             for col in set(relations).intersection(paramkeys):
                 submodel = get_related_model(self.model, col)
 
-                if type(params[col]) == list:
+                if type(data[col]) == list:
                     # model has several related objects
-                    for subparams in params[col]:
+                    for subparams in data[col]:
                         subinst = get_or_create(self.session, submodel,
                                                 subparams)
                         try:
@@ -1219,19 +1222,25 @@ class API(ModelView):
                 else:
                     # model has single related object
                     subinst = get_or_create(self.session, submodel,
-                                            params[col])
+                                            data[col])
                     setattr(instance, col, subinst)
 
             # add the created model to the session
             self.session.add(instance)
             self.session.commit()
+            # Get the dictionary representation of the new instance.
             result = self._inst_to_dict(instance)
-
-            primary_key = str(result[primary_key_name(instance)])
+            # Determine the value of the primary key for this instance and
+            # encode URL-encode it (in case it is a Unicode string).
+            pk_name = self.primary_key or primary_key_name(instance)
+            primary_key = result[pk_name]
+            try:
+                primary_key = str(primary_key)
+            except UnicodeEncodeError:
+                primary_key = url_quote_plus(primary_key.encode('utf-8'))
 
             # The URL at which a client can access the newly created instance
             # of the model.
-
             url = '{0}/{1}'.format(request.base_url, primary_key)
             # Provide that URL in the Location header in the response.
             headers = dict(Location=url)
@@ -1245,7 +1254,7 @@ class API(ModelView):
         except (DataError, IntegrityError, ProgrammingError) as exception:
             self.session.rollback()
             current_app.logger.exception(str(exception))
-            return dict(message=str(exception)), 400
+            return dict(message=type(exception).__name__), 400
 
     def patch(self, instid, relationname, relationinstid):
         """Updates the instance specified by ``instid`` of the named model, or
@@ -1325,12 +1334,17 @@ class API(ModelView):
                 return dict(message='Unable to construct query'), 400
         else:
             # create a SQLAlchemy Query which has exactly the specified row
-            query = query_by_primary_key(self.session, self.model, instid)
+            query = query_by_primary_key(self.session, self.model, instid,
+                                         self.primary_key)
             if query.count() == 0:
                 return {_STATUS: 404}, 404
             assert query.count() == 1, 'Multiple rows with same ID'
 
-        relations = self._update_relations(query, data)
+        try:
+            relations = self._update_relations(query, data)
+        except self.validation_exceptions as exception:
+            current_app.logger.exception(str(exception))
+            return self._handle_validation_exception(exception)
         field_list = frozenset(data) ^ relations
         data = dict((field, data[field]) for field in field_list)
 
@@ -1352,7 +1366,7 @@ class API(ModelView):
             return self._handle_validation_exception(exception)
         except (DataError, IntegrityError, ProgrammingError) as exception:
             current_app.logger.exception(str(exception))
-            return dict(message=str(exception)), 400
+            return dict(message=type(exception).__name__), 400
 
         # Perform any necessary postprocessing.
         if patchmany:
